--
-- DE2 (Cyclone-II) Entity for Interactive Project Game
-- Authors:
--      Abdulhamid Ghandour
--      Thomas John
--      Jaime Peretzman
--      Bharadwaj Vellore
--
-- Desc:
-- Avalon Interface for the Vision Block.
-- readdata has X co-ord in lower 16 bits
--          has Y co-ord in next  15 bits
--          has no_detect in MSB
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity avalon_vision is

port (
	-- Avalon Signals
	clk          : in std_logic;
	reset_n      : in std_logic;
	address      : in unsigned(4 downto 0);
	write        : in std_logic;
	read         : in std_logic;
	chipselect   : in std_logic;
	readdata     : out unsigned (31 downto 0);
	writedata    : in unsigned (31 downto 0);
	
	-- Camera Signals
	master_clk   : out std_logic;
	pixel_clk    : in std_logic;
	line_valid   : in std_logic;
	frame_valid  : in std_logic;
	pixel_data   : in unsigned (9 downto 0);
	
	-- Board Signals
	threshold    : in unsigned (9 downto 0);  -- SW9 to SW0
	no_detect    : out std_logic ;             -- LEDG0
	cal_direction: out unsigned (6 downto 0);
	vision_flags : out unsigned (7 downto 0)
	);
end avalon_vision;

architecture toplevel of avalon_vision is

signal ram_address : unsigned(4 downto 0);
signal data_signal : unsigned (9 downto 0);
signal valid_green_signal : std_logic;
signal valid_green_cropped_signal : std_logic;
signal end_of_frame_signal : std_logic;
signal end_of_row_signal : std_logic;
signal end_of_row_cropped_signal : std_logic;
signal no_detect_signal : std_logic;
signal x_1_signal,x_2_signal,y_1_signal,y_2_signal   : unsigned  (10 downto 0);
signal calibration_on_signal  : std_logic;
signal calibration_on_signal_int  : std_logic;
signal repos_signal : unsigned (6 downto 0);
signal xout_signal : unsigned (15 downto 0);
signal yout_signal : unsigned (14 downto 0);
signal reset_int : std_logic := '0';
signal sandboxStartX_signal : unsigned(31 downto 0) := (others => '0');
signal sandboxEndX_signal : unsigned(31 downto 0) := (others => '1');
signal sandboxStartY_signal : unsigned(31 downto 0) := (others => '0');
signal sandboxEndY_signal : unsigned(31 downto 0) := (others => '1');
signal green_column_thr_signal : unsigned ( 9 downto 0) := "0100101100";
signal green_row_thr_signal      : unsigned (9 downto 0) := "0110010000";
component ci_pxl port(
    clk             : in std_logic;
    mclk            : out std_logic; 	-- Master CLK to Camera
    lval            : in std_logic;     -- Line Valid from Camera
    fval            : in std_logic;     -- Frame Valid from Camera
    pixclk          : in std_logic;     -- Pixel CLK from Camera
    datain          : in unsigned(9 downto 0);    -- Pixel Data from Camera
    dataout         : out unsigned(9 downto 0);
    valid_green     : out std_logic;
    end_of_frame    : out std_logic;
    end_of_row      : out std_logic;
    sandboxStartX   : in unsigned(31 downto 0);
    sandboxStartY   : in unsigned(31 downto 0);
    sandboxEndX     : in unsigned(31 downto 0);
    sandboxEndY     : in unsigned(31 downto 0)
   );
end component;

component visionsystem port(
	clk : in std_logic;
	pixel_data : in unsigned (9 downto 0);
	valid_green : in std_logic;
	endofrow : in std_logic;
	endofframe : in std_logic;
	threshold : in unsigned (9 downto 0);
	xout : out unsigned (15 downto 0);
	yout : out unsigned (15 downto 0);
    led0, led1, led2, led3, led4, led5, led6, led7 : out std_logic;
	no_detect : out std_logic
    );
end component;

component calibration port (
    reset      : in std_logic;
    clk	       : in std_logic; 
    valid_green : in std_logic;
    end_row 	: in std_logic;
    end_frame   : in std_logic;
    green_pixel_value : in unsigned (9 downto 0);
--    green_column_thr   : in unsigned (9 downto 0);
--    green_row_thr      : in unsigned (9 downto 0);
    repos       : out unsigned (6 downto 0) := "1000000";
    x_1         : out unsigned  (10 downto 0) := "00000000000";
    y_1         : out unsigned  (10 downto 0) := "00000000000";
    x_2         : out unsigned  (10 downto 0) := "00000000000";
    y_2         : out unsigned  (10 downto 0) := "00000000000";
    calibration_on : in std_logic;
    threshold   : in unsigned (9 downto 0);
    leds : out unsigned(6 downto 0)
  );
end component;

component imagecropper port (
    clk	            : in std_logic; 
    valid_green_in  : in std_logic;
    valid_green_out : out std_logic;
    end_row_in 	    : in std_logic;
    end_row_out     : out std_logic; 
    end_frame       : in std_logic;
    crop_start_x    : in unsigned  (10 downto 0);
    crop_end_x      : in unsigned  (10 downto 0);
    crop_start_y    : in unsigned  (10 downto 0);
    crop_end_y      : in unsigned  (10 downto 0)
  );
end component;

begin
    ram_address <= address;
    reset_int <= not reset_n;
	CAMERA: ci_pxl port map(
		clk => clk,
		mclk => master_clk,
		lval => line_valid,
		fval => frame_valid,
		pixclk => pixel_clk,
		datain => pixel_data,
		dataout => data_signal,
		valid_green => valid_green_signal,
		end_of_frame => end_of_frame_signal,
		end_of_row => end_of_row_signal,
		sandboxStartX => sandboxStartX_signal,
		sandboxEndX => sandboxEndX_signal,
		sandboxStartY => sandboxStartY_signal,
		sandboxEndY => sandboxEndY_signal
		);
		
	VISION: visionsystem port map(
		clk => clk,
		pixel_data => data_signal,
		valid_green => valid_green_cropped_signal,
		endofrow => end_of_row_cropped_signal,
		endofframe => end_of_frame_signal,
		threshold => threshold,
		xout => xout_signal,
		yout (14 downto 0) => yout_signal,
		no_detect => no_detect_signal,
		led0 => vision_flags(0),
		led1 => vision_flags(1),
		led2 => vision_flags(2),
		led3 => vision_flags(3),
		led4 => vision_flags(4),
		led5 => vision_flags(5),
		led6 => vision_flags(6),
		led7 => vision_flags(7)
		);
		
    calibrator: calibration port map(
      reset       => '0',
      clk         => clk,
      valid_green => valid_green_cropped_signal,
      end_row     => end_of_row_cropped_signal,
      end_frame   => end_of_frame_signal,
      green_pixel_value => data_signal,
--	  green_column_thr => green_column_thr_signal,
--	  green_row_thr => green_column_thr_signal,
      repos       => repos_signal,
      x_1         => x_1_signal,
      y_1         => y_1_signal, 
      x_2         => x_2_signal, 
      y_2         => y_2_signal, 
      calibration_on => calibration_on_signal,
      threshold   => threshold,
      leds(5 downto 0) => cal_direction(5 downto 0)
    );

    CROPPER: imagecropper port map(
      clk	          => clk,
      valid_green_in  => valid_green_signal,
      valid_green_out => valid_green_cropped_signal,
      end_row_in 	  => end_of_row_signal,
      end_row_out     => end_of_row_cropped_signal,
      end_frame       => end_of_frame_signal,
      crop_start_x    => sandboxStartX_signal(10 downto 0),
      crop_end_x      => sandboxEndX_signal(10 downto 0),
      crop_start_y    => sandboxStartY_signal(10 downto 0),
      crop_end_y      => sandboxEndY_signal(10 downto 0)
    );

    host_control: process (clk)
    begin
      if rising_edge(clk) then
        if chipselect = '1' then
          if write = '1' then		
            if ram_address = 4 then
              calibration_on_signal_int <= writedata(0);
            elsif ram_address = 5 then
              sandboxStartX_signal <= writedata;
            elsif ram_address = 6 then
              sandboxEndX_signal <= writedata;
            elsif ram_address = 7 then
              sandboxStartY_signal <= writedata;
            elsif ram_address = 8 then
              sandboxEndY_signal <= writedata;
--            elsif ram_address = 9 then
--			  green_column_thr_signal <= writedata( 9 downto 0);
--            elsif ram_address = 10 then
--			  green_row_thr_signal <= writedata( 9 downto 0);			
            end if;			
          end if;

          if read = '1' then	
            if ram_address = 0 then
              readdata(30 downto 0) <= yout_signal & xout_signal;
              readdata(31) <= no_detect_signal;
            elsif ram_address = 1 then
              readdata(10 downto 0) <= x_1_signal;
              readdata(21 downto 11) <= y_1_signal;
              readdata(31 downto 22) <= (others => '0');
            elsif ram_address = 2 then
              readdata(10 downto 0) <= x_2_signal;
              readdata(21 downto 11) <= y_2_signal;
              readdata(31 downto 22) <= (others => '0');
            elsif ram_address = 3 then
              readdata <= ("0000000000000000000000000" & repos_signal);
            end if;
          end if;
        end if;-- end of chipselect
      end if;
    end process host_control;

	no_detect <= no_detect_signal;
	calibration_on_signal <= calibration_on_signal_int;
	cal_direction(6) <= valid_green_signal;
end toplevel;

		
		
