library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity de2_wm8731_audio is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    test_mode : in std_logic;       --    Audio CODEC controller test mode
    audio_request : out std_logic;  --    Audio controller request new data
    data : in unsigned(15 downto 0);
  
    -- Audio interface signals
    AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
    AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  de2_wm8731_audio;

architecture rtl of de2_wm8731_audio is     

    signal lrck : std_logic;
    signal bclk : std_logic;
    signal xck  : std_logic;
    
    signal lrck_divider : unsigned(7 downto 0); 
    signal bclk_divider : unsigned(3 downto 0);
    
    signal set_bclk : std_logic;
    signal set_lrck : std_logic;
    signal clr_bclk : std_logic;
    signal lrck_lat : std_logic;
    
    signal shift_out : unsigned(15 downto 0);

    signal sin_out     : unsigned(7 downto 0);
    signal sin_counter : unsigned(11 downto 0);    
    signal audio_on    : std_logic;
begin
  
    -- LRCK divider 
    -- Audio chip main clock is 18.432MHz / Sample rate 48KHz
    -- Divider is 18.432 MHz / 48KHz = 192 (X"C0")
    -- Left justify mode set by I2C controller
    
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then 
        lrck_divider <= (others => '0');
      elsif lrck_divider = X"BF"  then        -- "C0" minus 1
        lrck_divider <= X"00";
      else 
        lrck_divider <= lrck_divider + 1;
      end if;
    end if;   
  end process;

  process (clk)
  begin
    if rising_edge(clk) then
      if audio_on = '0' then
        audio_on <= test_mode;
      end if;
      if sin_counter = x"36E" then
        audio_on <= '0';
      end if;
    end if;
  end process;

  process (clk)
  begin
    if rising_edge(clk) then      
      if reset_n = '0' then 
        bclk_divider <= (others => '0');
      elsif bclk_divider = X"B" or set_lrck = '1'  then  
        bclk_divider <= X"0";
      else 
        bclk_divider <= bclk_divider + 1;
      end if;
    end if;
  end process;

  set_lrck <= '1' when lrck_divider = X"BF" else '0';
    
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        lrck <= '0';
      elsif set_lrck = '1' then 
        lrck <= not lrck;
      end if;
    end if;
  end process;
    
  -- BCLK divider
  set_bclk <= '1' when bclk_divider(3 downto 0) = "0101" else '0';
  clr_bclk <= '1' when bclk_divider(3 downto 0) = "1011" else '0';
  
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        bclk <= '0';
      elsif set_lrck = '1' or clr_bclk = '1' then
        bclk <= '0';
      elsif set_bclk = '1' then 
        bclk <= '1';
      end if;
    end if;
  end process;

  -- Audio data shift output
  process (clk)
  begin
    if rising_edge(clk) then
      if reset_n = '0' then
        shift_out <= (others => '0');
      elsif set_lrck = '1' then
        if audio_on = '1' then 
          shift_out <= ("00" & sin_out & "000000");
        else 
          shift_out <= data;
        end if;
      elsif clr_bclk = '1' then 
        shift_out <= shift_out (14 downto 0) & '0';
      end if;
    end if;   
  end process;

    -- Audio outputs
    
    AUD_ADCLRCK  <= lrck;          
    AUD_DACLRCK  <= lrck;          
    AUD_DACDAT   <= shift_out(15); 
    AUD_BCLK     <= bclk;          

    -- Self test with Sin wave
    
    process(clk)      
    begin
      if rising_edge(clk) then
        if reset_n = '0' then 
            sin_counter <= (others => '0');
        elsif lrck_lat = '1' and lrck = '0'  then  
          if sin_counter = x"36E" then 
            sin_counter <= x"000";
          elsif audio_on = '1' then  
            sin_counter <= sin_counter + 1;
          end if;
        end if;
      end if;
    end process;

    process(clk)
    begin
      if rising_edge(clk) then
        lrck_lat <= lrck;
      end if;
    end process;

    process (clk) 
    begin
      if rising_edge(clk) then 
        if lrck_lat = '1' and lrck = '0' then
          audio_request <= '1';
        else 
          audio_request <= '0';
        end if;
      end if;
    end process;

  with sin_counter select sin_out <=
     x"49" when x"001",
	 x"52" when x"002",
	 x"46" when x"003",
	 x"46" when x"004",
	 x"03" when x"005",
	 x"66" when x"006",
	 x"00" when x"007",
	 x"00" when x"008",
	 x"04" when x"009",
	 x"57" when x"00A",
	 x"45" when x"00B",
	 x"56" when x"00C",
	 x"6d" when x"00D",
	 x"06" when x"00E",
	 x"20" when x"00F",
	 x"74" when x"010",
	 x"00" when x"011",
	 x"10" when x"012",
	 x"00" when x"013",
	 x"00" when x"014",
	 x"00" when x"015",
	 x"01" when x"016",
	 x"00" when x"017",
	 x"01" when x"018",
	 x"bb" when x"019",
	 x"80" when x"01A",
	 x"00" when x"01B",
	 x"00" when x"01C",
	 x"bb" when x"01D",
	 x"80" when x"01E",
	 x"00" when x"01F",
	 x"00" when x"020",
	 x"00" when x"021",
	 x"01" when x"022",
	 x"00" when x"023",
	 x"08" when x"024",
	 x"61" when x"025",
	 x"64" when x"026",
	 x"61" when x"027",
	 x"74" when x"028",
	 x"02" when x"029",
	 x"ea" when x"02A",
	 x"00" when x"02B",
	 x"00" when x"02C",
	 x"8b" when x"02D",
	 x"90" when x"02E",
	 x"7d" when x"02F",
	 x"86" when x"030",
	 x"71" when x"031",
	 x"76" when x"032",
	 x"6c" when x"033",
	 x"6b" when x"034",
	 x"75" when x"035",
	 x"6e" when x"036",
	 x"8f" when x"037",
	 x"82" when x"038",
	 x"a9" when x"039",
	 x"9e" when x"03A",
	 x"a2" when x"03B",
	 x"aa" when x"03C",
	 x"73" when x"03D",
	 x"8f" when x"03E",
	 x"3a" when x"03F",
	 x"57" when x"040",
	 x"1f" when x"041",
	 x"26" when x"042",
	 x"32" when x"043",
	 x"21" when x"044",
	 x"71" when x"045",
	 x"4e" when x"046",
	 x"d3" when x"047",
	 x"a0" when x"048",
	 x"fc" when x"049",
	 x"f7" when x"04A",
	 x"fd" when x"04B",
	 x"f6" when x"04C",
	 x"b8" when x"04D",
	 x"f3" when x"04E",
	 x"26" when x"04F",
	 x"68" when x"050",
	 x"04" when x"051",
	 x"06" when x"052",
	 x"03" when x"053",
	 x"08" when x"054",
	 x"29" when x"055",
	 x"08" when x"056",
	 x"91" when x"057",
	 x"5f" when x"058",
	 x"dc" when x"059",
	 x"bb" when x"05A",
	 x"e7" when x"05B",
	 x"ea" when x"05C",
	 x"bd" when x"05D",
	 x"d7" when x"05E",
	 x"94" when x"05F",
	 x"a6" when x"060",
	 x"86" when x"061",
	 x"89" when x"062",
	 x"81" when x"063",
	 x"84" when x"064",
	 x"76" when x"065",
	 x"7e" when x"066",
	 x"6c" when x"067",
	 x"6f" when x"068",
	 x"6f" when x"069",
	 x"6b" when x"06A",
	 x"78" when x"06B",
	 x"74" when x"06C",
	 x"80" when x"06D",
	 x"7e" when x"06E",
	 x"84" when x"06F",
	 x"82" when x"070",
	 x"84" when x"071",
	 x"84" when x"072",
	 x"83" when x"073",
	 x"85" when x"074",
	 x"82" when x"075",
	 x"83" when x"076",
	 x"7e" when x"077",
	 x"80" when x"078",
	 x"74" when x"079",
	 x"7a" when x"07A",
	 x"66" when x"07B",
	 x"6d" when x"07C",
	 x"5c" when x"07D",
	 x"61" when x"07E",
	 x"5a" when x"07F",
	 x"5a" when x"080",
	 x"60" when x"081",
	 x"5c" when x"082",
	 x"6f" when x"083",
	 x"67" when x"084",
	 x"86" when x"085",
	 x"79" when x"086",
	 x"9d" when x"087",
	 x"92" when x"088",
	 x"ab" when x"089",
	 x"a6" when x"08A",
	 x"ad" when x"08B",
	 x"ae" when x"08C",
	 x"a5" when x"08D",
	 x"aa" when x"08E",
	 x"94" when x"08F",
	 x"9d" when x"090",
	 x"7d" when x"091",
	 x"89" when x"092",
	 x"6b" when x"093",
	 x"73" when x"094",
	 x"65" when x"095",
	 x"65" when x"096",
	 x"6c" when x"097",
	 x"67" when x"098",
	 x"79" when x"099",
	 x"72" when x"09A",
	 x"8c" when x"09B",
	 x"82" when x"09C",
	 x"a0" when x"09D",
	 x"96" when x"09E",
	 x"ab" when x"09F",
	 x"a8" when x"0A0",
	 x"a5" when x"0A1",
	 x"aa" when x"0A2",
	 x"8f" when x"0A3",
	 x"9b" when x"0A4",
	 x"74" when x"0A5",
	 x"82" when x"0A6",
	 x"5a" when x"0A7",
	 x"67" when x"0A8",
	 x"3f" when x"0A9",
	 x"4d" when x"0AA",
	 x"2a" when x"0AB",
	 x"33" when x"0AC",
	 x"23" when x"0AD",
	 x"24" when x"0AE",
	 x"35" when x"0AF",
	 x"29" when x"0B0",
	 x"5d" when x"0B1",
	 x"47" when x"0B2",
	 x"90" when x"0B3",
	 x"76" when x"0B4",
	 x"c5" when x"0B5",
	 x"ab" when x"0B6",
	 x"ea" when x"0B7",
	 x"db" when x"0B8",
	 x"f6" when x"0B9",
	 x"f4" when x"0BA",
	 x"e4" when x"0BB",
	 x"f0" when x"0BC",
	 x"be" when x"0BD",
	 x"d3" when x"0BE",
	 x"8f" when x"0BF",
	 x"a7" when x"0C0",
	 x"67" when x"0C1",
	 x"79" when x"0C2",
	 x"50" when x"0C3",
	 x"58" when x"0C4",
	 x"4b" when x"0C5",
	 x"4c" when x"0C6",
	 x"51" when x"0C7",
	 x"4d" when x"0C8",
	 x"59" when x"0C9",
	 x"55" when x"0CA",
	 x"63" when x"0CB",
	 x"5d" when x"0CC",
	 x"6f" when x"0CD",
	 x"69" when x"0CE",
	 x"79" when x"0CF",
	 x"75" when x"0D0",
	 x"7f" when x"0D1",
	 x"7c" when x"0D2",
	 x"82" when x"0D3",
	 x"81" when x"0D4",
	 x"81" when x"0D5",
	 x"82" when x"0D6",
	 x"7d" when x"0D7",
	 x"7f" when x"0D8",
	 x"7e" when x"0D9",
	 x"7c" when x"0DA",
	 x"84" when x"0DB",
	 x"80" when x"0DC",
	 x"8a" when x"0DD",
	 x"88" when x"0DE",
	 x"8c" when x"0DF",
	 x"8b" when x"0E0",
	 x"8d" when x"0E1",
	 x"8d" when x"0E2",
	 x"90" when x"0E3",
	 x"8e" when x"0E4",
	 x"92" when x"0E5",
	 x"91" when x"0E6",
	 x"95" when x"0E7",
	 x"94" when x"0E8",
	 x"93" when x"0E9",
	 x"94" when x"0EA",
	 x"87" when x"0EB",
	 x"8f" when x"0EC",
	 x"75" when x"0ED",
	 x"7e" when x"0EE",
	 x"67" when x"0EF",
	 x"6d" when x"0F0",
	 x"63" when x"0F1",
	 x"64" when x"0F2",
	 x"67" when x"0F3",
	 x"64" when x"0F4",
	 x"71" when x"0F5",
	 x"6c" when x"0F6",
	 x"7c" when x"0F7",
	 x"77" when x"0F8",
	 x"84" when x"0F9",
	 x"80" when x"0FA",
	 x"88" when x"0FB",
	 x"86" when x"0FC",
	 x"8c" when x"0FD",
	 x"89" when x"0FE",
	 x"8f" when x"0FF",
	 x"8e" when x"100",
	 x"8d" when x"101",
	 x"8f" when x"102",
	 x"80" when x"103",
	 x"88" when x"104",
	 x"6f" when x"105",
	 x"76" when x"106",
	 x"65" when x"107",
	 x"69" when x"108",
	 x"67" when x"109",
	 x"64" when x"10A",
	 x"78" when x"10B",
	 x"6e" when x"10C",
	 x"90" when x"10D",
	 x"84" when x"10E",
	 x"a4" when x"10F",
	 x"9b" when x"110",
	 x"aa" when x"111",
	 x"a9" when x"112",
	 x"a0" when x"113",
	 x"a7" when x"114",
	 x"8b" when x"115",
	 x"96" when x"116",
	 x"77" when x"117",
	 x"81" when x"118",
	 x"6b" when x"119",
	 x"70" when x"11A",
	 x"6a" when x"11B",
	 x"69" when x"11C",
	 x"6f" when x"11D",
	 x"6c" when x"11E",
	 x"74" when x"11F",
	 x"72" when x"120",
	 x"74" when x"121",
	 x"75" when x"122",
	 x"71" when x"123",
	 x"73" when x"124",
	 x"71" when x"125",
	 x"70" when x"126",
	 x"76" when x"127",
	 x"73" when x"128",
	 x"80" when x"129",
	 x"7b" when x"12A",
	 x"88" when x"12B",
	 x"83" when x"12C",
	 x"90" when x"12D",
	 x"8c" when x"12E",
	 x"94" when x"12F",
	 x"93" when x"130",
	 x"92" when x"131",
	 x"94" when x"132",
	 x"8b" when x"133",
	 x"90" when x"134",
	 x"7e" when x"135",
	 x"85" when x"136",
	 x"6c" when x"137",
	 x"75" when x"138",
	 x"5e" when x"139",
	 x"64" when x"13A",
	 x"5e" when x"13B",
	 x"5c" when x"13C",
	 x"6d" when x"13D",
	 x"64" when x"13E",
	 x"80" when x"13F",
	 x"77" when x"140",
	 x"92" when x"141",
	 x"8a" when x"142",
	 x"9d" when x"143",
	 x"98" when x"144",
	 x"9e" when x"145",
	 x"9f" when x"146",
	 x"96" when x"147",
	 x"9a" when x"148",
	 x"8a" when x"149",
	 x"90" when x"14A",
	 x"7d" when x"14B",
	 x"84" when x"14C",
	 x"71" when x"14D",
	 x"77" when x"14E",
	 x"69" when x"14F",
	 x"6c" when x"150",
	 x"6a" when x"151",
	 x"69" when x"152",
	 x"75" when x"153",
	 x"6f" when x"154",
	 x"83" when x"155",
	 x"7c" when x"156",
	 x"8f" when x"157",
	 x"89" when x"158",
	 x"96" when x"159",
	 x"94" when x"15A",
	 x"8f" when x"15B",
	 x"94" when x"15C",
	 x"7b" when x"15D",
	 x"86" when x"15E",
	 x"62" when x"15F",
	 x"6d" when x"160",
	 x"55" when x"161",
	 x"59" when x"162",
	 x"5c" when x"163",
	 x"56" when x"164",
	 x"72" when x"165",
	 x"65" when x"166",
	 x"88" when x"167",
	 x"7e" when x"168",
	 x"91" when x"169",
	 x"8f" when x"16A",
	 x"8d" when x"16B",
	 x"90" when x"16C",
	 x"8a" when x"16D",
	 x"8a" when x"16E",
	 x"92" when x"16F",
	 x"8c" when x"170",
	 x"a3" when x"171",
	 x"9b" when x"172",
	 x"b0" when x"173",
	 x"ab" when x"174",
	 x"a9" when x"175",
	 x"af" when x"176",
	 x"8c" when x"177",
	 x"9d" when x"178",
	 x"64" when x"179",
	 x"79" when x"17A",
	 x"45" when x"17B",
	 x"52" when x"17C",
	 x"43" when x"17D",
	 x"40" when x"17E",
	 x"60" when x"17F",
	 x"4f" when x"180",
	 x"8d" when x"181",
	 x"76" when x"182",
	 x"b1" when x"183",
	 x"a1" when x"184",
	 x"bd" when x"185",
	 x"bb" when x"186",
	 x"ad" when x"187",
	 x"b8" when x"188",
	 x"89" when x"189",
	 x"9c" when x"18A",
	 x"67" when x"18B",
	 x"76" when x"18C",
	 x"59" when x"18D",
	 x"5d" when x"18E",
	 x"63" when x"18F",
	 x"5c" when x"190",
	 x"72" when x"191",
	 x"6b" when x"192",
	 x"76" when x"193",
	 x"76" when x"194",
	 x"70" when x"195",
	 x"74" when x"196",
	 x"67" when x"197",
	 x"6b" when x"198",
	 x"65" when x"199",
	 x"65" when x"19A",
	 x"6e" when x"19B",
	 x"68" when x"19C",
	 x"82" when x"19D",
	 x"77" when x"19E",
	 x"97" when x"19F",
	 x"8c" when x"1A0",
	 x"a3" when x"1A1",
	 x"9e" when x"1A2",
	 x"a2" when x"1A3",
	 x"a5" when x"1A4",
	 x"97" when x"1A5",
	 x"9d" when x"1A6",
	 x"88" when x"1A7",
	 x"8f" when x"1A8",
	 x"7e" when x"1A9",
	 x"82" when x"1AA",
	 x"7d" when x"1AB",
	 x"7d" when x"1AC",
	 x"81" when x"1AD",
	 x"7e" when x"1AE",
	 x"84" when x"1AF",
	 x"83" when x"1B0",
	 x"82" when x"1B1",
	 x"84" when x"1B2",
	 x"7e" when x"1B3",
	 x"81" when x"1B4",
	 x"7b" when x"1B5",
	 x"7d" when x"1B6",
	 x"79" when x"1B7",
	 x"79" when x"1B8",
	 x"7a" when x"1B9",
	 x"79" when x"1BA",
	 x"83" when x"1BB",
	 x"7e" when x"1BC",
	 x"95" when x"1BD",
	 x"8b" when x"1BE",
	 x"a2" when x"1BF",
	 x"9c" when x"1C0",
	 x"9d" when x"1C1",
	 x"a2" when x"1C2",
	 x"82" when x"1C3",
	 x"92" when x"1C4",
	 x"5b" when x"1C5",
	 x"70" when x"1C6",
	 x"3b" when x"1C7",
	 x"48" when x"1C8",
	 x"37" when x"1C9",
	 x"34" when x"1CA",
	 x"54" when x"1CB",
	 x"43" when x"1CC",
	 x"77" when x"1CD",
	 x"68" when x"1CE",
	 x"7a" when x"1CF",
	 x"7c" when x"1D0",
	 x"61" when x"1D1",
	 x"6e" when x"1D2",
	 x"56" when x"1D3",
	 x"59" when x"1D4",
	 x"7f" when x"1D5",
	 x"65" when x"1D6",
	 x"d8" when x"1D7",
	 x"a5" when x"1D8",
	 x"fb" when x"1D9",
	 x"f7" when x"1DA",
	 x"f7" when x"1DB",
	 x"f9" when x"1DC",
	 x"f3" when x"1DD",
	 x"fc" when x"1DE",
	 x"75" when x"1DF",
	 x"bc" when x"1E0",
	 x"0a" when x"1E1",
	 x"38" when x"1E2",
	 x"0a" when x"1E3",
	 x"02" when x"1E4",
	 x"08" when x"1E5",
	 x"03" when x"1E6",
	 x"52" when x"1E7",
	 x"28" when x"1E8",
	 x"98" when x"1E9",
	 x"79" when x"1EA",
	 x"b2" when x"1EB",
	 x"ac" when x"1EC",
	 x"9d" when x"1ED",
	 x"ab" when x"1EE",
	 x"75" when x"1EF",
	 x"8a" when x"1F0",
	 x"51" when x"1F1",
	 x"62" when x"1F2",
	 x"43" when x"1F3",
	 x"46" when x"1F4",
	 x"54" when x"1F5",
	 x"47" when x"1F6",
	 x"80" when x"1F7",
	 x"68" when x"1F8",
	 x"af" when x"1F9",
	 x"9a" when x"1FA",
	 x"c8" when x"1FB",
	 x"c0" when x"1FC",
	 x"c3" when x"1FD",
	 x"c9" when x"1FE",
	 x"a7" when x"1FF",
	 x"b6" when x"200",
	 x"88" when x"201",
	 x"97" when x"202",
	 x"71" when x"203",
	 x"7b" when x"204",
	 x"66" when x"205",
	 x"6a" when x"206",
	 x"67" when x"207",
	 x"65" when x"208",
	 x"6b" when x"209",
	 x"69" when x"20A",
	 x"70" when x"20B",
	 x"6e" when x"20C",
	 x"74" when x"20D",
	 x"72" when x"20E",
	 x"76" when x"20F",
	 x"75" when x"210",
	 x"77" when x"211",
	 x"76" when x"212",
	 x"7d" when x"213",
	 x"79" when x"214",
	 x"87" when x"215",
	 x"82" when x"216",
	 x"8e" when x"217",
	 x"8b" when x"218",
	 x"8c" when x"219",
	 x"8e" when x"21A",
	 x"83" when x"21B",
	 x"88" when x"21C",
	 x"79" when x"21D",
	 x"7e" when x"21E",
	 x"72" when x"21F",
	 x"74" when x"220",
	 x"77" when x"221",
	 x"73" when x"222",
	 x"87" when x"223",
	 x"7e" when x"224",
	 x"95" when x"225",
	 x"8f" when x"226",
	 x"96" when x"227",
	 x"98" when x"228",
	 x"8a" when x"229",
	 x"91" when x"22A",
	 x"7f" when x"22B",
	 x"84" when x"22C",
	 x"79" when x"22D",
	 x"7c" when x"22E",
	 x"76" when x"22F",
	 x"78" when x"230",
	 x"74" when x"231",
	 x"75" when x"232",
	 x"73" when x"233",
	 x"73" when x"234",
	 x"75" when x"235",
	 x"74" when x"236",
	 x"7b" when x"237",
	 x"78" when x"238",
	 x"84" when x"239",
	 x"7f" when x"23A",
	 x"90" when x"23B",
	 x"8a" when x"23C",
	 x"9a" when x"23D",
	 x"96" when x"23E",
	 x"9e" when x"23F",
	 x"9d" when x"240",
	 x"99" when x"241",
	 x"9c" when x"242",
	 x"89" when x"243",
	 x"92" when x"244",
	 x"73" when x"245",
	 x"7f" when x"246",
	 x"60" when x"247",
	 x"68" when x"248",
	 x"58" when x"249",
	 x"5a" when x"24A",
	 x"5d" when x"24B",
	 x"59" when x"24C",
	 x"6f" when x"24D",
	 x"64" when x"24E",
	 x"8c" when x"24F",
	 x"7c" when x"250",
	 x"a7" when x"251",
	 x"9b" when x"252",
	 x"af" when x"253",
	 x"ae" when x"254",
	 x"a1" when x"255",
	 x"ab" when x"256",
	 x"80" when x"257",
	 x"92" when x"258",
	 x"59" when x"259",
	 x"6c" when x"25A",
	 x"3b" when x"25B",
	 x"48" when x"25C",
	 x"3c" when x"25D",
	 x"37" when x"25E",
	 x"5d" when x"25F",
	 x"49" when x"260",
	 x"8d" when x"261",
	 x"75" when x"262",
	 x"b9" when x"263",
	 x"a5" when x"264",
	 x"d1" when x"265",
	 x"c8" when x"266",
	 x"cd" when x"267",
	 x"d3" when x"268",
	 x"b0" when x"269",
	 x"c1" when x"26A",
	 x"86" when x"26B",
	 x"9b" when x"26C",
	 x"60" when x"26D",
	 x"71" when x"26E",
	 x"4c" when x"26F",
	 x"54" when x"270",
	 x"4d" when x"271",
	 x"4a" when x"272",
	 x"62" when x"273",
	 x"56" when x"274",
	 x"7d" when x"275",
	 x"70" when x"276",
	 x"90" when x"277",
	 x"89" when x"278",
	 x"94" when x"279",
	 x"94" when x"27A",
	 x"8b" when x"27B",
	 x"90" when x"27C",
	 x"7d" when x"27D",
	 x"84" when x"27E",
	 x"73" when x"27F",
	 x"77" when x"280",
	 x"71" when x"281",
	 x"71" when x"282",
	 x"79" when x"283",
	 x"74" when x"284",
	 x"84" when x"285",
	 x"7e" when x"286",
	 x"8d" when x"287",
	 x"8a" when x"288",
	 x"8e" when x"289",
	 x"8e" when x"28A",
	 x"89" when x"28B",
	 x"8c" when x"28C",
	 x"82" when x"28D",
	 x"86" when x"28E",
	 x"7e" when x"28F",
	 x"80" when x"290",
	 x"80" when x"291",
	 x"7e" when x"292",
	 x"85" when x"293",
	 x"82" when x"294",
	 x"88" when x"295",
	 x"87" when x"296",
	 x"87" when x"297",
	 x"88" when x"298",
	 x"81" when x"299",
	 x"84" when x"29A",
	 x"77" when x"29B",
	 x"7c" when x"29C",
	 x"71" when x"29D",
	 x"74" when x"29E",
	 x"71" when x"29F",
	 x"70" when x"2A0",
	 x"76" when x"2A1",
	 x"73" when x"2A2",
	 x"7d" when x"2A3",
	 x"79" when x"2A4",
	 x"86" when x"2A5",
	 x"82" when x"2A6",
	 x"8b" when x"2A7",
	 x"89" when x"2A8",
	 x"89" when x"2A9",
	 x"8b" when x"2AA",
	 x"81" when x"2AB",
	 x"86" when x"2AC",
	 x"7a" when x"2AD",
	 x"7d" when x"2AE",
	 x"75" when x"2AF",
	 x"77" when x"2B0",
	 x"74" when x"2B1",
	 x"75" when x"2B2",
	 x"76" when x"2B3",
	 x"75" when x"2B4",
	 x"79" when x"2B5",
	 x"78" when x"2B6",
	 x"7b" when x"2B7",
	 x"7a" when x"2B8",
	 x"7a" when x"2B9",
	 x"7b" when x"2BA",
	 x"7a" when x"2BB",
	 x"7a" when x"2BC",
	 x"7b" when x"2BD",
	 x"7a" when x"2BE",
	 x"7f" when x"2BF",
	 x"7c" when x"2C0",
	 x"86" when x"2C1",
	 x"82" when x"2C2",
	 x"8d" when x"2C3",
	 x"8a" when x"2C4",
	 x"91" when x"2C5",
	 x"8f" when x"2C6",
	 x"91" when x"2C7",
	 x"92" when x"2C8",
	 x"8e" when x"2C9",
	 x"90" when x"2CA",
	 x"88" when x"2CB",
	 x"8c" when x"2CC",
	 x"81" when x"2CD",
	 x"85" when x"2CE",
	 x"7a" when x"2CF",
	 x"7d" when x"2D0",
	 x"75" when x"2D1",
	 x"77" when x"2D2",
	 x"71" when x"2D3",
	 x"72" when x"2D4",
	 x"6d" when x"2D5",
	 x"6f" when x"2D6",
	 x"6e" when x"2D7",
	 x"6d" when x"2D8",
	 x"74" when x"2D9",
	 x"70" when x"2DA",
	 x"7a" when x"2DB",
	 x"77" when x"2DC",
	 x"7f" when x"2DD",
	 x"7d" when x"2DE",
	 x"82" when x"2DF",
	 x"81" when x"2E0",
	 x"81" when x"2E1",
	 x"82" when x"2E2",
	 x"7d" when x"2E3",
	 x"7f" when x"2E4",
	 x"7a" when x"2E5",
	 x"7b" when x"2E6",
	 x"7e" when x"2E7",
	 x"7b" when x"2E8",
	 x"86" when x"2E9",
	 x"81" when x"2EA",
	 x"8f" when x"2EB",
	 x"8b" when x"2EC",
	 x"92" when x"2ED",
	 x"91" when x"2EE",
	 x"8d" when x"2EF",
	 x"90" when x"2F0",
	 x"86" when x"2F1",
	 x"89" when x"2F2",
	 x"82" when x"2F3",
	 x"83" when x"2F4",
	 x"86" when x"2F5",
	 x"83" when x"2F6",
	 x"8c" when x"2F7",
	 x"89" when x"2F8",
	 x"8c" when x"2F9",
	 x"8d" when x"2FA",
	 x"82" when x"2FB",
	 x"88" when x"2FC",
	 x"73" when x"2FD",
	 x"7b" when x"2FE",
	 x"69" when x"2FF",
	 x"6d" when x"300",
	 x"6b" when x"301",
	 x"68" when x"302",
	 x"78" when x"303",
	 x"71" when x"304",
	 x"86" when x"305",
	 x"7f" when x"306",
	 x"8b" when x"307",
	 x"8a" when x"308",
	 x"85" when x"309",
	 x"8a" when x"30A",
	 x"7b" when x"30B",
	 x"80" when x"30C",
	 x"73" when x"30D",
	 x"76" when x"30E",
	 x"73" when x"30F",
	 x"72" when x"310",
	 x"7a" when x"311",
	 x"76" when x"312",
	 x"83" when x"313",
	 x"7f" when x"314",
	 x"8a" when x"315",
	 x"87" when x"316",
	 x"49" when x"317",
	 x"4c" when x"318",
	 x"54" when x"319",
	 x"53" when x"31A",
	 x"00" when x"31B",
	 x"50" when x"31C",
	 x"00" when x"31D",
	 x"00" when x"31E",
	 x"4e" when x"31F",
	 x"49" when x"320",
	 x"4f" when x"321",
	 x"46" when x"322",
	 x"43" when x"323",
	 x"49" when x"324",
	 x"44" when x"325",
	 x"52" when x"326",
	 x"00" when x"327",
	 x"0c" when x"328",
	 x"00" when x"329",
	 x"00" when x"32A",
	 x"30" when x"32B",
	 x"32" when x"32C",
	 x"38" when x"32D",
	 x"30" when x"32E",
	 x"30" when x"32F",
	 x"2d" when x"330",
	 x"2d" when x"331",
	 x"35" when x"332",
	 x"39" when x"333",
	 x"30" when x"334",
	 x"00" when x"335",
	 x"00" when x"336",
	 x"45" when x"337",
	 x"49" when x"338",
	 x"47" when x"339",
	 x"4e" when x"33A",
	 x"00" when x"33B",
	 x"11" when x"33C",
	 x"00" when x"33D",
	 x"00" when x"33E",
	 x"61" when x"33F",
	 x"4a" when x"340",
	 x"6d" when x"341",
	 x"69" when x"342",
	 x"20" when x"343",
	 x"65" when x"344",
	 x"65" when x"345",
	 x"50" when x"346",
	 x"65" when x"347",
	 x"72" when x"348",
	 x"7a" when x"349",
	 x"74" when x"34A",
	 x"61" when x"34B",
	 x"6d" when x"34C",
	 x"00" when x"34D",
	 x"6e" when x"34E",
	 x"01" when x"34F",
	 x"00" when x"350",
	 x"53" when x"351",
	 x"49" when x"352",
	 x"54" when x"353",
	 x"46" when x"354",
	 x"00" when x"355",
	 x"16" when x"356",
	 x"00" when x"357",
	 x"00" when x"358",
	 x"6f" when x"359",
	 x"53" when x"35A",
	 x"79" when x"35B",
	 x"6e" when x"35C",
	 x"53" when x"35D",
	 x"20" when x"35E",
	 x"75" when x"35F",
	 x"6f" when x"360",
	 x"64" when x"361",
	 x"6e" when x"362",
	 x"46" when x"363",
	 x"20" when x"364",
	 x"72" when x"365",
	 x"6f" when x"366",
	 x"65" when x"367",
	 x"67" when x"368",
	 x"38" when x"369",
	 x"20" when x"36A",
	 x"30" when x"36B",
	 x"2e" when x"36C",
	 x"00" when x"36D",
	 x"00" when x"36E",
    X"00" when others;      

end architecture;


